@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2003 Marti Maria
+//  Copyright (C) 1998-2007 Marti Maria
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -25,7 +25,14 @@
 // #define ICM_COMPARATIVE      1
 // #define CHECK_SPEED          1
 
+#ifdef __BORLANDC__
+#     include <condefs.h>
+#endif
+
 #include "lcms.h"
+
+
+
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +41,7 @@
 #include <icm.h>
 #endif
 
+
 #define PREC  20
 
 #define TYPE_XYZA_16            (COLORSPACE_SH(PT_XYZ)|CHANNELS_SH(3)|BYTES_SH(2)|EXTRA_SH(1))
@@ -41,6 +49,9 @@
 
 typedef struct {BYTE r, g, b, a;}   Scanline_rgb1;
 typedef struct {WORD r, g, b, a;}   Scanline_rgb2;
+typedef struct {BYTE r, g, b;}      Scanline_rgb8;
+typedef struct {WORD r, g, b;}      Scanline_rgb0;
+
 
 // Print a dot for gauging
 
@@ -99,9 +110,9 @@ int CheckSwab(void)
         return 1;
 #endif
 
-    swab(Test, Test, 6);
+    swab((char*) Test, (char*) Test, 6);
 
-    if (strncmp(Test, "\x2\x1\x4\x3\x6\x5", 6) != 0)
+    if (strncmp((char*) Test, "\x2\x1\x4\x3\x6\x5", 6) != 0)
     {
             printf("\nOOOPPSS! swab() does not work as expected in your machine!\n\n");
             printf("Please, edit lcms.h and uncomment the USE_CUSTOM_SWAB toggle.\n");
@@ -111,6 +122,25 @@ int CheckSwab(void)
     return 1;
 }
 
+
+static
+int CheckQuickFloor(void)
+{
+        
+    if ((_cmsQuickFloor(1.234) != 1) ||
+        (_cmsQuickFloor(32767.234) != 32767) ||
+        (_cmsQuickFloor(-1.234) != -2) ||
+        (_cmsQuickFloor(-32767.1) != -32768)) {
+
+                printf("\nOOOPPSS! _cmsFloor() does not work as expected in your machine!\n\n");
+                printf("Please, edit lcms.h and uncomment the LCMS_DEFAULT_FLOOR_CONVERSION toggle.\n");
+                return 0;
+
+    }
+
+    return 1;
+}
+
 typedef struct _Stats {
                       double n, x, y, x2, y2, xy;
                       double Peak;
@@ -142,10 +172,14 @@ void PrintStatistics(clock_t atime, LPSTATS Stats)
        // These are statistics of 16 bit, so divide
        // by 257 to get dE relative to 8 bits
 
-       printf("\ndE: mean=%g, SD=%g, max=%g ",
+       printf("\n");
+
+       if (Stats) 
+          printf("dE: mean=%g, SD=%g, max=%g ",
                      (Stats->x / Stats -> n) / 257.,
                      (Std(Stats)) / 257.,
-                     Stats -> Peak / 257.);
+                     Stats -> Peak / 257.);       
+           
 
        if (atime > 0)
             printf("[%d tics, %g sec.]", (int) diff, a);
@@ -158,24 +192,19 @@ void PrintStatistics(clock_t atime, LPSTATS Stats)
 static
 void TestFixedPoint(void)
 {
-       Fixed32 a, b, c, d, e;
+       Fixed32 a, b, c, d; 
        double f;
 
-       a = DOUBLE_TO_FIXED(1.12345);
-       b = DOUBLE_TO_FIXED(2.56789);
-
+       a = DOUBLE_TO_FIXED(1.1234);
+       b = DOUBLE_TO_FIXED(2.5678);
+       
        c = FixedMul(a, b);
-       e = FixedDiv(a, b);
+      
+       d = FIXED_REST_TO_INT(c);
+       f = ((double) d / 0xffff) * 1000000.0;
 
-       d = (c & 0xffff);
-       f = ((double) d / 0xffff) * 10000.0;
-
-       printf("Testing fixed point: 2.8848960205 = %d.%d\n", FIXED_TO_INT(c), (int) f);
-
-       d = (e & 0xffff);
-       f = ((double) d / 0xffff) * 10000.0;
-
-       printf("\t\t0.437499269828536 = %d.%d\n", FIXED_TO_INT(e), (int) f);
+       printf("Testing fixed point:\t%f = %d.%d\n", (1.1234 * 2.5678), FIXED_TO_INT(c), (int) f);
+      
 }
 
 
@@ -221,7 +250,7 @@ static
 int TestJointCurves(void)
 {
     LPGAMMATABLE Forward, Reverse, Result;
-    BOOL rc;
+    LCMSBOOL rc;
 
     printf("Testing curves join ...");
 
@@ -247,6 +276,7 @@ int TestJointCurves(void)
 }
 
 
+
 // Check reversing of gamma curves
 
 #define NPOINTS     1024
@@ -267,7 +297,7 @@ int TestReversingOfCurves(void)
     Reverse = cmsBuildGamma(NPOINTS, 1.0/3.0);
 
     Computed = cmsReverseGamma(NPOINTS, Gamma);
-    
+
     for (i=0; i < NPOINTS; i++) {
 
             dE = fabs(Reverse->GammaTable[i] - Computed->GammaTable[i]);
@@ -275,17 +305,19 @@ int TestReversingOfCurves(void)
             Stats.x += dE;                                   
             Stats.x2 += (dE * dE);
             Stats.n += 1.0;
-            if (dE > Stats.Peak) 
-                Stats.Peak = dE;
+            if (dE > Stats.Peak) {
+                Stats.Peak = dE;                                
+            }
 
-            if (dE > 0x1100) {
-                printf("Coarse error! %x", (int) dE);
+            if (dE > 0x0010) {
+                printf("Coarse error! %x on entry %d: %X/%X", (int) dE, i, Reverse->GammaTable[i],
+                                                                           Computed->GammaTable[i]);
                 return 0;
             }
                                    
     }
-
-    PrintStatistics(0, &Stats); 
+    
+    if (Stats.Peak > 0) PrintStatistics(0, &Stats); 
     printf(" pass.\n");
     cmsFreeGamma(Gamma);
     cmsFreeGamma(Reverse);
@@ -301,7 +333,7 @@ static
 int TestLinearInterpolation(int lExhaustive)
 {
        static WORD Tab[4098];
-       int j, i, k;
+       int j, i, k = 0;
        L16PARAMS p;
        int n;
        clock_t time;
@@ -530,11 +562,11 @@ int TestLinearInterpolation(int lExhaustive)
 
        if (!lExhaustive) return 1;
 
-       printf("Now, checking interpolation errors for tables of n elements ...\n");
+       printf("Now, testing interpolation errors for tables of n elements ...\n");
 
        for (j=10; j < 4096; j ++)
        {
-       printf("%d\r", j);
+       if ((j % 10) == 0) printf("%d\r", j);
 
        for (i=0; i <= j; i++)
               {
@@ -549,9 +581,9 @@ int TestLinearInterpolation(int lExhaustive)
               if (n != i) k++;
 
        }
-       if (k > 0) printf("\r%d: %d errors\n", j, k);
+       
        }
-
+       printf("\n%d: %d errors\n\n", j, k);
        return 1;
 }
 
@@ -561,7 +593,8 @@ static
 int IsGood(const char *frm, WORD in, WORD out)
 {
 
-        if ((abs(in - out) > 2)) {
+        // 1 for rounding
+        if ((abs(in - out) > 1)) {
 
               printf("error %s %x - %x\n", frm, in, out);
               return 0;
@@ -571,7 +604,7 @@ int IsGood(const char *frm, WORD in, WORD out)
 }
 
 static
-BOOL TestReverseLinearInterpolation(void)
+LCMSBOOL TestReverseLinearInterpolation(void)
 {
         WORD Tab[20];
         L16PARAMS p;
@@ -582,7 +615,7 @@ BOOL TestReverseLinearInterpolation(void)
 
         cmsCalcL16Params(16, &p);
 
-        for (i=0; i < 16; i++) Tab[i] = i * 0x1111;
+        for (i=0; i < 16; i++) Tab[i] = (WORD) i * 0x1111;
 
         printf("\ton normal monotonic curve...");
         for (i=0; i < 16; i++)
@@ -705,9 +738,11 @@ int Test3D(void)
 
        cmsEvalLUT(MyLut, In, Out);
 
-      // Check results, I will tolerate error <= 1  for rounding
 
-       // printf("%x, %x\n", In[0], Out[0]);
+       
+        // Check results, I will tolerate error <= 1  for rounding
+
+       
 
        if (!IsGood("Channel 1", Out[0], In[0])) return 0;
        if (!IsGood("Channel 2", Out[1], (WORD) ((double) In[1] / 2))) return 0;
@@ -715,6 +750,8 @@ int Test3D(void)
 
       }
 
+   
+   
    // Last, remember free stuff
 
    cmsFreeLUT(MyLut);
@@ -745,7 +782,7 @@ void PrintMatrix(LPMAT3 lpM)
 
 
 static
-BOOL CmpMatrix(LPMAT3 lpM1, LPMAT3 lpM2, double tolerance)
+LCMSBOOL CmpMatrix(LPMAT3 lpM1, LPMAT3 lpM2, double tolerance)
 {
        int i, j;
 
@@ -763,7 +800,7 @@ BOOL CmpMatrix(LPMAT3 lpM1, LPMAT3 lpM2, double tolerance)
 
 
 static
-BOOL TestMatrixCreation(void)
+LCMSBOOL TestMatrixCreation(void)
 {
        MAT3 Mat;
        int rc;
@@ -876,8 +913,8 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv, int MaxErr)
        clock_t t;
 
 
-       bin  = (Scanline_rgb2 *) malloc(256*sizeof(Scanline_rgb2));
-       bout = (Scanline_rgb2 *) malloc(256*sizeof(Scanline_rgb2));
+       bin  = (Scanline_rgb2 *) _cmsMalloc(256*sizeof(Scanline_rgb2));
+       bout = (Scanline_rgb2 *) _cmsMalloc(256*sizeof(Scanline_rgb2));
 
 
        ClearStats(&Stats);
@@ -896,9 +933,10 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv, int MaxErr)
 
                             for (b=0; b < 256; b++)
                             {
-                            bin[b].r = r << 8;          // For L 0nly to 0xFF00
+                            bin[b].r = (WORD) r << 8;          // For L 0nly to 0xFF00
                             bin[b].g = RGB_8_TO_16(g);
                             bin[b].b = RGB_8_TO_16(b);
+                            bin[b].a = 0;
                             }
 
                             cmsDoTransform(xform, bin, bout, 256);
@@ -922,8 +960,8 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv, int MaxErr)
                                           printf("Coarse error! : In=(%x,%x,%x) Out=(%x,%x,%x)\n",
                                                         bin[b].r, bin[b].g, bin[b].b,
                                                         bout[b].r, bout[b].g, bout[b].b);
-                                          free(bin);
-                                          free(bout);
+                                          _cmsFree(bin);
+                                          _cmsFree(bout);
                                           return 0;
                                    }
                             }
@@ -934,8 +972,8 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv, int MaxErr)
 
 
        PrintStatistics(t, &Stats);
-       free(bin);
-       free(bout);
+       _cmsFree(bin);
+       _cmsFree(bout);
 
        return 1;
 }
@@ -960,7 +998,7 @@ int TestInducedError(DWORD Type)
                                   INTENT_RELATIVE_COLORIMETRIC, 0);
               
 
-       nMaxError = TestFullSpectrum(xform, 31, 0x1000L);
+       nMaxError = TestFullSpectrum(xform, 31, 0x800);
 
        printf("\n");
 
@@ -998,7 +1036,7 @@ double Convertab(WORD v)
 
 static
 int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, 
-                      int nRedInterv, int lWithStats, BOOL lIsLab)
+                      int nRedInterv, int lWithStats, LCMSBOOL lIsLab)
 {
        int r, g, b;
        double err;
@@ -1007,30 +1045,31 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2,
        int OutOfGamut = 0;
 
 
-       bin   = (Scanline_rgb2 *) malloc(256*sizeof(Scanline_rgb2));
-       bout1 = (Scanline_rgb2 *) malloc(256*sizeof(Scanline_rgb2));
-       bout2 = (Scanline_rgb2 *) malloc(256*sizeof(Scanline_rgb2));
+       bin   = (Scanline_rgb2 *) _cmsMalloc(256*sizeof(Scanline_rgb2));
+       bout1 = (Scanline_rgb2 *) _cmsMalloc(256*sizeof(Scanline_rgb2));
+       bout2 = (Scanline_rgb2 *) _cmsMalloc(256*sizeof(Scanline_rgb2));
 
 
        ClearStats(&Stats);
        Stats.x = 0.0; Stats.n = 0.0; // GCC BUG HERE!!!!
 
 
-       for (r=0; r < BASE; r+= nRedInterv)
+       for (r=0; r <= BASE; r+= nRedInterv)
        {
               // printf("\r%02x:", r);
 
               Dot();
-              for (g=0; g < BASE; g++)
+              for (g=0; g <= BASE; g++)
                      {
                             // I will test random LSB
 
-                            for (b=0; b < BASE; b++)     // 256
+                            for (b=0; b <= BASE; b++)     // 256
                             {
 
                             bin[b].r = RGB_8_TO_16(r);
                             bin[b].g = RGB_8_TO_16(g);
                             bin[b].b = RGB_8_TO_16(b);
+                            bin[b].a = 0;
                             }
 
                             cmsDoTransform(xform1, bin, bout1, 256);
@@ -1038,7 +1077,7 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2,
 
                             // I'm using b as index
 
-                            for (b=0; b < BASE; b ++) {
+                            for (b=0; b <= BASE; b ++) {
 
                                    // I measure the error using vector distance
                                    // Only if encodable values
@@ -1096,9 +1135,9 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2,
                 printf("Out of encodeable representation=%d\n\n", OutOfGamut);
      
 
-       free(bin);
-       free(bout1);
-       free(bout2);
+       _cmsFree(bin);
+       _cmsFree(bout1);
+       _cmsFree(bout2);
 
        return 1;
 }
@@ -1106,7 +1145,7 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2,
 
 
 static
-BOOL CheckXYZ(LPcmsCIEXYZ Check, double X, double Y, double Z)
+LCMSBOOL CheckXYZ(LPcmsCIEXYZ Check, double X, double Y, double Z)
 {
     return ((fabs(Check->X - X) < 0.001) && 
             (fabs(Check->Y - Y) < 0.001) &&
@@ -1129,7 +1168,7 @@ LPGAMMATABLE Build_sRGBGamma(void)
 }
 
 static
-BOOL Check_sRGBGamma(LPGAMMATABLE Shape)
+LCMSBOOL Check_sRGBGamma(LPGAMMATABLE Shape)
 {
     LPGAMMATABLE sRGB = Build_sRGBGamma();
     int i;
@@ -1359,7 +1398,7 @@ int TestMultiprofile(void)
 
     hXForm = cmsCreateMultiprofileTransform(Profiles, 7, TYPE_RGBA_16, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, cmsFLAGS_HIGHRESPRECALC);
     
-    printf("Checking multiprofile transforms (6 profiles)");
+    printf("Testing multiprofile transforms (6 profiles)");
 
     nMaxErr = TestFullSpectrum(hXForm, 31, 0x1000L);
 
@@ -1384,7 +1423,7 @@ int TestLinearizationDevicelink()
     cmsHPROFILE Profiles[10];
     int nMaxErr;
 
-    printf("Checking linearization devicelink");
+    printf("Testing linearization devicelink");
 
     Transfer[0] = cmsBuildGamma(256, 1./2.2);
     Transfer[1] = cmsBuildGamma(256, 1./2.2);
@@ -1427,6 +1466,48 @@ int TestLinearizationDevicelink()
 }
 
 
+static
+int TestLinearizationDevicelink2()
+{
+    LPGAMMATABLE Transfer[3];
+    cmsHPROFILE hLin1;
+    cmsHTRANSFORM hXForm;    
+    int nMaxErr;
+
+    printf("Testing saved linearization devicelink");
+
+    Transfer[0] = cmsBuildGamma(256, 1);
+    Transfer[1] = cmsBuildGamma(256, 1);
+    Transfer[2] = cmsBuildGamma(256, 1);
+    
+    hLin1 = cmsCreateLinearizationDeviceLink(icSigRgbData, Transfer);
+
+    _cmsSaveProfile(hLin1, "lin1.icc");
+    cmsFreeGammaTriple(Transfer);
+    cmsCloseProfile(hLin1);
+
+    hLin1 = cmsOpenProfileFromFile("lin1.icc", "r");
+
+    hXForm = cmsCreateTransform(hLin1, TYPE_RGBA_16, NULL, TYPE_RGBA_16, INTENT_ABSOLUTE_COLORIMETRIC, 0);
+
+    if (!hXForm) {
+
+        printf("Error!\n");
+        return 1;
+    }
+    
+    nMaxErr = TestFullSpectrum(hXForm, 31, 1);
+
+    cmsDeleteTransform(hXForm);
+    cmsCloseProfile(hLin1);
+    unlink("lin1.icc");
+
+    printf("\n");
+
+    return nMaxErr;
+}
+
+
 static
 int TestDeviceLinkGeneration()
 {
@@ -1435,7 +1516,7 @@ int TestDeviceLinkGeneration()
     int nMaxErr;
 
 
-    printf("Checking devicelink generation");
+    printf("Testing devicelink generation");
 
     hsRGB     = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "r");
     hIdentity = cmsCreateTransform(hsRGB, TYPE_RGBA_16, hsRGB, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0);
@@ -1445,6 +1526,7 @@ int TestDeviceLinkGeneration()
     cmsCloseProfile(hDevLink);
     cmsCloseProfile(hsRGB);
     cmsDeleteTransform(hIdentity);
+	
 
     hDevLink = cmsOpenProfileFromFile("devicelink.icm", "r");
     hXForm   = cmsCreateTransform(hDevLink, TYPE_RGBA_16, NULL, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0); 
@@ -1459,6 +1541,56 @@ int TestDeviceLinkGeneration()
     return nMaxErr;
 }
 
+static
+int TestInkLimiting()
+{
+    cmsHPROFILE hIL;
+    cmsHTRANSFORM hXForm;
+    BYTE In[4], Out[4];
+    int i, j, k, l, res;
+    
+
+
+    printf("Testing ink limiting ");
+    
+    hIL = cmsCreateInkLimitingDeviceLink(icSigCmykData, 100);
+
+    
+    hXForm = cmsCreateTransform(hIL, TYPE_CMYK_8, NULL, TYPE_CMYK_8, INTENT_RELATIVE_COLORIMETRIC, 0);
+    if (!hXForm) {
+
+        printf("Error!\n");
+        return 0;
+    }
+
+    for (l=0; l < 255; l += 8) {
+        Dot();      
+        for (k=0; k < 255; k += 8) 
+            for (j=0; j < 255; j += 8) 
+                for (i=0; i < 255; i += 8) {
+
+                    In[0] = (BYTE) i; In[1] = (BYTE) j; In[2] = (BYTE) k; In[3] = (BYTE) l;
+
+                    cmsDoTransform(hXForm, In, Out, 1);
+
+                    res = Out[0] + Out[1] + Out[2] + Out[3];
+                        
+                    if (res > 0x100) {
+            
+                        printf("Failed! (%d) \n", res);
+                        return 0;   
+                    }
+        }
+    }
+
+    cmsDeleteTransform(hXForm);
+    cmsCloseProfile(hIL);
+    printf(" pass.\n");
+
+    return 1;
+}
+
+
 
 static
 void CheckPlanar(void)
@@ -1492,13 +1624,15 @@ void CheckPlanar(void)
 #ifndef NON_WINDOWS
 
 static
-void CompareWithICM(void)
+void CompareWithICM_16bit(void)
 {
 
     HTRANSFORM hICMxform;
     HPROFILE   hICMProfileFrom, hICMProfileTo;
     LOGCOLORSPACE LogColorSpace;
     COLOR In, Out;
+	COLOR *InBlk, *OutBlk, *InPtr;
+	size_t size;
     int r, g, b;
     PROFILE Profile;
     clock_t atime;
@@ -1507,17 +1641,17 @@ void CompareWithICM(void)
     cmsHTRANSFORM hlcmsxform;
 
 
-    printf("\n\nComparative with MS-Windows ICM:\n");
+    printf("\n\nComparative with MS-Windows ICM (16 bits per sample):\n");
     
 
     Profile.dwType = PROFILE_FILENAME;
-    Profile.pProfileData = "sRGBSpac.ICM";
-    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
 
     hICMProfileFrom = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
-    Profile.pProfileData = "sRGBSpac.ICM";
-    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
     hICMProfileTo   = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
     ZeroMemory(&LogColorSpace, sizeof(LOGCOLORSPACE));
@@ -1525,7 +1659,107 @@ void CompareWithICM(void)
     LogColorSpace.lcsSignature = LCS_SIGNATURE;
     LogColorSpace.lcsVersion   = 0x400;
     LogColorSpace.lcsCSType    = LCS_CALIBRATED_RGB;
-    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.ICM");
+    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.icm");
+
+    hICMxform = CreateColorTransform(&LogColorSpace, hICMProfileTo, NULL, BEST_MODE);
+
+
+
+	size = 256 * 256 * 256;
+	InBlk = _cmsMalloc((size_t) size * sizeof(COLOR));
+	OutBlk = _cmsMalloc((size_t) size * sizeof(COLOR));
+
+	if (InBlk == NULL || OutBlk == NULL) {
+		printf("Out of memory\n"); exit(2);
+	}
+			
+    printf("Windows ICM is transforming full spectrum...");
+
+	InPtr = InBlk;
+	for (r=0; r < 255; r++)
+        for (g=0; g < 255; g++)
+            for (b=0; b < 255; b++) {
+
+        InPtr->rgb.red   = (r << 8) | r;
+        InPtr->rgb.green = (g << 8) | g;
+        InPtr->rgb.blue  = (b << 8) | b;
+
+		InPtr++;
+	}
+
+    atime = clock();
+    
+    TranslateColors( hICMxform, InBlk, size, COLOR_RGB, OutBlk, COLOR_RGB);
+    
+    diff = clock() - atime;
+    seconds = (double) diff / CLOCKS_PER_SEC;
+
+
+    printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
+  
+    CloseColorProfile(hICMProfileFrom);
+    CloseColorProfile(hICMProfileTo);
+    DeleteColorTransform(hICMxform);
+
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
+
+    printf("lcms is transforming full spectrum...");
+
+    atime = clock();
+
+    cmsDoTransform(hlcmsxform, InBlk, OutBlk, size);
+   
+    diff = clock() - atime;
+    seconds = (double) diff / CLOCKS_PER_SEC;
+
+    printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
+
+    cmsDeleteTransform(hlcmsxform);
+    cmsCloseProfile(hlcmsProfileIn);
+    cmsCloseProfile(hlcmsProfileOut);
+
+	_cmsFree(InBlk);
+	_cmsFree(OutBlk);
+}
+
+static
+void CompareWithICM_8bit(void)
+{
+
+    HTRANSFORM hICMxform;
+    HPROFILE   hICMProfileFrom, hICMProfileTo;
+    LOGCOLORSPACE LogColorSpace;
+    RGBQUAD In, Out;
+    int r, g, b;
+    PROFILE Profile;
+    clock_t atime;
+    double seconds, diff;
+    cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
+    cmsHTRANSFORM hlcmsxform;
+
+
+    printf("\n\nComparative with MS-Windows ICM (8 bits per sample):\n");
+    
+
+    Profile.dwType = PROFILE_FILENAME;
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
+
+    hICMProfileFrom = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
+
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
+    hICMProfileTo   = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
+
+    ZeroMemory(&LogColorSpace, sizeof(LOGCOLORSPACE));
+
+    LogColorSpace.lcsSignature = LCS_SIGNATURE;
+    LogColorSpace.lcsVersion   = 0x400;
+    LogColorSpace.lcsCSType    = LCS_CALIBRATED_RGB;
+    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.icm");
 
     hICMxform = CreateColorTransform(&LogColorSpace, hICMProfileTo, NULL, BEST_MODE);
 
@@ -1537,11 +1771,13 @@ void CompareWithICM(void)
         for (g=0; g < 255; g++)
             for (b=0; b < 255; b++) {
 
-        In.rgb.red   = (r << 8) | r;
-        In.rgb.green = (g << 8) | g;
-        In.rgb.blue  = (b << 8) | b;
-
-        TranslateColors( hICMxform, &In, 1, COLOR_RGB, &Out, COLOR_RGB);
+        In.rgbRed   = r;
+        In.rgbGreen = g;
+        In.rgbBlue  = b;
+        
+        if (!TranslateBitmapBits(hICMxform, &In,  BM_RGBTRIPLETS, 1, 1, 0, &Out, BM_RGBTRIPLETS, 0, NULL, 0))
+            exit(2);
+       
     }
 
     diff = clock() - atime;
@@ -1554,10 +1790,10 @@ void CompareWithICM(void)
     CloseColorProfile(hICMProfileTo);
     DeleteColorTransform(hICMxform);
 
-    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
-    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
 
-    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_BGRA_8, hlcmsProfileOut, TYPE_BGRA_8, INTENT_PERCEPTUAL, 0);
 
     printf("lcms is transforming full spectrum...");
 
@@ -1567,11 +1803,11 @@ void CompareWithICM(void)
         for (g=0; g < 255; g++)
             for (b=0; b < 255; b++) {
 
-        In.rgb.red   = (r << 8) | r;
-        In.rgb.green = (g << 8) | g;
-        In.rgb.blue  = (b << 8) | b;
-
-        cmsDoTransform(hlcmsxform, &In.rgb, &Out.rgb, 1);
+                In.rgbRed   = r;
+                In.rgbGreen = g;
+                In.rgbBlue  = b;
+                
+        cmsDoTransform(hlcmsxform, &In, &Out, 1);
     }
 
     diff = clock() - atime;
@@ -1585,49 +1821,117 @@ void CompareWithICM(void)
 
 }
 
+
 #endif
 #endif
 
 
 #ifdef CHECK_SPEED
+
+
+
 static
 void SpeedTest(void)
 {
 
-    int r, g, b;
+    int r, g, b, j;
     clock_t atime;
     double seconds, diff;
     cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
     cmsHTRANSFORM hlcmsxform;
-    COLOR In, Out;
-   
-   
-    printf("\n\nRaw speed check:\n");
+    Scanline_rgb0 *In;
+	size_t Mb;
+   	
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
 
-    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.ICM", "r");
-    hlcmsProfileOut = cmsOpenProfileFromFile("sRGB Color Space Profile.ICM", "r");
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, cmsFLAGS_NOTCACHE);
+
+	Mb = 256*256*256*sizeof(Scanline_rgb0);
+
+	In = (Scanline_rgb0*) _cmsMalloc(Mb);
+
+	j = 0;
+	for (r=0; r < 256; r++)
+        for (g=0; g < 256; g++)
+            for (b=0; b < 256; b++) {
+
+        In[j].r = (WORD) ((r << 8) | r);
+        In[j].g = (WORD) ((g << 8) | g);
+        In[j].b = (WORD) ((b << 8) | b);
+
+		j++;
+	}
 
-    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
 
     printf("lcms is transforming full spectrum...");
 
     atime = clock();
 
-    for (r=0; r < 255; r++)
-        for (g=0; g < 255; g++)
-            for (b=0; b < 255; b++) {
-
-        In.rgb.red   = (r << 8) | r;
-        In.rgb.green = (g << 8) | g;
-        In.rgb.blue  = (b << 8) | b;
-
-        cmsDoTransform(hlcmsxform, &In.rgb, &Out.rgb, 1);
-    }
+    cmsDoTransform(hlcmsxform, In, In, 256*256*256);
 
     diff = clock() - atime;
     seconds = (double) diff / CLOCKS_PER_SEC;
+	_cmsFree(In);
+	
     
-    printf("done.\n[%d tics, %g sec, %g Mb/sec.]\n", (int) diff, seconds, 32. / seconds );
+    printf("done.\n[%d tics, %g sec, %g Mpixel/sec.]\n", (int) diff, seconds, Mb / (1024*1024*seconds*3*2) );
+
+    cmsDeleteTransform(hlcmsxform);
+    cmsCloseProfile(hlcmsProfileIn);
+    cmsCloseProfile(hlcmsProfileOut);
+
+}
+
+
+
+static
+void SpeedTest2(void)
+{
+
+    int r, g, b, j;
+    clock_t atime;
+    double seconds, diff;
+    cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
+    cmsHTRANSFORM hlcmsxform;
+    Scanline_rgb8 *In;
+	size_t Mb;
+   
+   
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_8, hlcmsProfileOut, TYPE_RGB_8, INTENT_PERCEPTUAL, cmsFLAGS_NOTCACHE);
+
+	Mb = 256*256*256*sizeof(Scanline_rgb8);
+
+	In = (Scanline_rgb8*) _cmsMalloc(Mb);
+
+	j = 0;
+	for (r=0; r < 256; r++)
+        for (g=0; g < 256; g++)
+            for (b=0; b < 256; b++) {
+
+        In[j].r = (BYTE) r;
+        In[j].g = (BYTE) g;
+        In[j].b = (BYTE) b;
+
+		j++;
+	}
+
+
+    printf("lcms is transforming full spectrum...");
+
+    atime = clock();
+
+    cmsDoTransform(hlcmsxform, In, In, 256*256*256);
+
+    diff = clock() - atime;
+    seconds = (double) diff / CLOCKS_PER_SEC;
+	_cmsFree(In);
+	
+    
+    printf("done.\n[%d tics, %g sec, %g Mpixels/sec.]\n", (int) diff, seconds, Mb / (1024*1024*seconds*3) );
 
     cmsDeleteTransform(hlcmsxform);
     cmsCloseProfile(hlcmsProfileIn);
@@ -1638,7 +1942,357 @@ void SpeedTest(void)
 #endif
 
 
+static
+int TestSaveToMem(void)
+{
+      void    *memPtr=0;
+      size_t  bytesNeeded=0;
+      int rc = FALSE;
+      cmsHPROFILE hProfile = cmsCreate_sRGBProfile();
 
+      printf("Testing save to memory: ");
+
+     // pass 1 - compute length
+      if (!_cmsSaveProfileToMem(hProfile, memPtr, &bytesNeeded)) {
+                printf("Failed!\n");
+                return FALSE;
+      }
+    // pass 2 - generate profile
+      if(!bytesNeeded) {
+            printf("Failed!\n");
+            return FALSE;
+      }
+
+    memPtr = _cmsMalloc(bytesNeeded);
+    if (_cmsSaveProfileToMem(hProfile, memPtr, &bytesNeeded)) {
+            
+
+        cmsHPROFILE newProfile = cmsOpenProfileFromMem(memPtr, (DWORD) bytesNeeded);        
+        const char* s = cmsTakeProductName(newProfile);
+
+        if (strncmp(s, "sRGB", 4) == 0) rc = TRUE;
+
+        cmsCloseProfile(newProfile);
+        _cmsFree(memPtr);
+        
+    }
+
+    cmsCloseProfile(hProfile);
+
+    printf (rc ? "pass.\n" : "failed!\n");
+    return rc;
+}
+
+
+
+static
+int TestNamedColor(void)
+{
+    LPcmsNAMEDCOLORLIST nc2;    
+    cmsHPROFILE hProfile, hDevicelink, hsRGB, hLab;
+    cmsHTRANSFORM xform, rgb2lab;    
+    int i;
+
+
+    printf("Testing Named color profiles: ");
+
+
+    hsRGB    = cmsCreate_sRGBProfile();
+    hLab     = cmsCreateLabProfile(NULL);
+    
+    rgb2lab = cmsCreateTransform(hsRGB, TYPE_RGB_16, hLab, TYPE_Lab_16, INTENT_PERCEPTUAL, cmsFLAGS_NOTPRECALC);
+
+    nc2 = cmsAllocNamedColorList(64);
+
+    nc2 ->ColorantCount = 3;
+    strcpy(nc2 ->Prefix, "prefix");
+    strcpy(nc2 ->Suffix, "suffix");
+
+    for (i=0; i < 64; i++) {
+
+        WORD vv = RGB_8_TO_16((i*4));
+        
+        nc2 ->List[i].DeviceColorant[0] = vv;
+        nc2 ->List[i].DeviceColorant[1] = vv;
+        nc2 ->List[i].DeviceColorant[2] = vv;
+
+        cmsDoTransform(rgb2lab, nc2 ->List[i].DeviceColorant, nc2 ->List[i].PCS, 1);        
+        
+        sprintf(nc2 ->List[i].Name, "Color #%d", i);
+
+    }
+
+    hProfile = cmsOpenProfileFromFile("named.icc", "w");
+
+    cmsSetDeviceClass(hProfile, icSigNamedColorClass);
+    cmsSetPCS(hProfile, icSigLabData);
+    cmsSetColorSpace(hProfile, icSigRgbData);
+
+    cmsAddTag(hProfile, icSigNamedColor2Tag, (void*) nc2);
+    cmsAddTag(hProfile, icSigMediaWhitePointTag, cmsD50_XYZ());
+    cmsCloseProfile(hProfile);
+
+    cmsFreeNamedColorList(nc2);
+
+
+    hProfile = cmsOpenProfileFromFile("named.icc", "r");
+    
+    xform = cmsCreateTransform(hProfile, TYPE_NAMED_COLOR_INDEX, NULL, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
+
+    for (i=0; i < 64; i++) {
+
+        WORD index;
+        WORD Color[3];
+
+        index = (WORD) i;
+
+        cmsDoTransform(xform, &index, Color, 1);
+
+        if (Color[0] != RGB_8_TO_16((i*4)) ||
+            Color[1] != RGB_8_TO_16((i*4)) ||
+            Color[2] != RGB_8_TO_16((i*4))) { 
+
+                    printf(" fail on spot color #%d\n", i); 
+                    return 0; 
+            }
+    }
+
+
+    cmsDeleteTransform(xform);
+    cmsCloseProfile(hProfile);
+    cmsDeleteTransform(rgb2lab);
+    cmsCloseProfile(hLab);
+
+        
+    hProfile = cmsOpenProfileFromFile("named.icc", "r");
+    
+    xform = cmsCreateTransform(hProfile, TYPE_NAMED_COLOR_INDEX, hsRGB, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
+
+    hDevicelink = cmsTransform2DeviceLink(xform, 0);
+
+    _cmsSaveProfile(hDevicelink, "named2.icc");
+    cmsCloseProfile(hDevicelink);
+    
+    cmsDeleteTransform(xform);
+    cmsCloseProfile(hProfile);
+    
+    cmsCloseProfile(hsRGB);
+        
+    unlink("named.icc");
+    unlink("named2.icc");
+
+    printf(" pass.\n");
+    return 1;
+}
+
+
+static
+int TestColorantTableTag()
+{
+	LPcmsNAMEDCOLORLIST	     nc2;
+	cmsHPROFILE hProfile = cmsOpenProfileFromFile("colTable.icc", "w");
+
+	nc2 = cmsAllocNamedColorList(3);
+
+	strcpy(nc2 ->List[0].Name, "Red");
+	strcpy(nc2 ->List[1].Name, "Green");
+	strcpy(nc2 ->List[2].Name, "Blue");
+
+	
+    cmsSetDeviceClass(hProfile, icSigOutputClass);
+    cmsSetPCS(hProfile, icSigLabData);
+    cmsSetColorSpace(hProfile, icSigRgbData);
+
+    cmsAddTag(hProfile, icSigColorantTableTag, (void*) nc2);
+    cmsAddTag(hProfile, icSigMediaWhitePointTag, cmsD50_XYZ());
+    cmsCloseProfile(hProfile);
+    cmsFreeNamedColorList(nc2);
+
+
+	hProfile = cmsOpenProfileFromFile("colTable.icc", "r");
+
+	nc2 = cmsReadColorantTable(hProfile, icSigColorantTableTag);
+
+	cmsFreeNamedColorList(nc2);
+	cmsCloseProfile(hProfile);
+
+	unlink("colTable.icc");
+
+	return 1;
+
+}
+
+// New to 1.13 -- CGATS/IT8.7
+
+
+#define NPOINTS_IT8 10  // (17*17*17*17)
+
+static
+int TestIT8(void)
+{
+    LCMSHANDLE it8;
+    int i;
+
+    printf("Testing CGATS parser: ");
+
+    it8 = cmsIT8Alloc();
+
+    cmsIT8SetSheetType(it8, "LCMS/TESTING");
+    cmsIT8SetPropertyStr(it8, "ORIGINATOR",   "1 2 3 4");
+    cmsIT8SetPropertyUncooked(it8, "DESCRIPTOR",   "1234");
+    cmsIT8SetPropertyStr(it8, "MANUFACTURER", "3");
+    cmsIT8SetPropertyDbl(it8, "CREATED",      4);
+    cmsIT8SetPropertyDbl(it8, "SERIAL",       5);
+    cmsIT8SetPropertyHex(it8, "MATERIAL",     0x123);
+
+    cmsIT8SetPropertyDbl(it8, "NUMBER_OF_SETS", NPOINTS_IT8);
+    cmsIT8SetPropertyDbl(it8, "NUMBER_OF_FIELDS", 4);
+
+    cmsIT8SetDataFormat(it8, 0, "SAMPLE_ID");
+    cmsIT8SetDataFormat(it8, 1, "RGB_R");
+    cmsIT8SetDataFormat(it8, 2, "RGB_G");
+    cmsIT8SetDataFormat(it8, 3, "RGB_B");
+
+    for (i=0; i < NPOINTS_IT8; i++) {
+
+          char Patch[20];
+
+          sprintf(Patch, "P%d", i);
+
+          cmsIT8SetDataRowCol(it8, i, 0, Patch);
+          cmsIT8SetDataRowColDbl(it8, i, 1, i);
+          cmsIT8SetDataRowColDbl(it8, i, 2, i);
+          cmsIT8SetDataRowColDbl(it8, i, 3, i);
+    }
+
+    cmsIT8SaveToFile(it8, "TEST.IT8");
+    cmsIT8Free(it8);
+
+
+	it8 = cmsIT8LoadFromFile("TEST.IT8");
+	cmsIT8SaveToFile(it8, "TEST.IT8");
+	cmsIT8Free(it8);
+
+
+
+    it8 = cmsIT8LoadFromFile("TEST.IT8");
+
+    if (cmsIT8GetPropertyDbl(it8, "DESCRIPTOR") != 1234) {
+    
+        printf("fail!\n");
+        return 0;
+    }
+
+
+	cmsIT8SetPropertyDbl(it8, "DESCRIPTOR", 5678);
+
+	if (cmsIT8GetPropertyDbl(it8, "DESCRIPTOR") != 5678) {
+    
+        printf("fail!\n");
+        return 0;
+    }
+
+
+    if (cmsIT8GetDataDbl(it8, "P3", "RGB_G") != 3) {
+        printf("fail!\n");
+        return 0;
+    }
+
+
+    cmsIT8Free(it8);
+
+    unlink("TEST.IT8");
+    printf("pass.\n");
+    return 1;
+
+}
+
+
+// Create CSA/CRD
+
+static
+void GenerateCSA(const char* cInProf)
+{
+	cmsHPROFILE hProfile;
+	
+	
+	DWORD n;
+	char* Buffer;
+
+
+	if (cInProf == NULL) 
+		hProfile = cmsCreateLabProfile(NULL);
+	else 
+		hProfile = cmsOpenProfileFromFile(cInProf, "r");
+
+	n = cmsGetPostScriptCSA(hProfile, 0, NULL, 0);
+	if (n == 0) return;
+
+	Buffer = (char*) _cmsMalloc(n + 1);
+	cmsGetPostScriptCSA(hProfile, 0, Buffer, n);
+	Buffer[n] = 0;
+	
+	_cmsFree(Buffer);
+	cmsCloseProfile(hProfile);
+}
+
+
+static
+void GenerateCRD(const char* cOutProf)
+{
+	cmsHPROFILE hProfile;
+	DWORD n;
+	char* Buffer;
+    DWORD dwFlags = 0;
+    
+
+	if (cOutProf == NULL) 
+		hProfile = cmsCreateLabProfile(NULL);
+	else 
+		hProfile = cmsOpenProfileFromFile(cOutProf, "r");
+
+	n = cmsGetPostScriptCRDEx(hProfile, 0, dwFlags, NULL, 0);
+	if (n == 0) return;
+
+	Buffer = (char*) _cmsMalloc(n + 1);
+    cmsGetPostScriptCRDEx(hProfile, 0, dwFlags, Buffer, n);
+	Buffer[n] = 0;
+	
+	_cmsFree(Buffer);
+	cmsCloseProfile(hProfile);
+}
+
+static 
+int TestPostScript()
+{
+	GenerateCSA("sRGB Color Space Profile.icm");
+	GenerateCRD("sRGB Color Space Profile.icm");
+	GenerateCSA(NULL);
+	GenerateCRD(NULL);
+
+	return 1;
+}
+
+
+static
+void TestLabFloat()
+{
+#define TYPE_LabA_DBL   (COLORSPACE_SH(PT_Lab)|CHANNELS_SH(3)|BYTES_SH(0)|EXTRA_SH(1)|DOSWAP_SH(1))
+
+	struct {
+		   double L, a, b;
+		   double A;
+	} a;
+	cmsCIELab b;
+	cmsHPROFILE hLab  = cmsCreateLabProfile(NULL);
+    cmsHTRANSFORM xform = cmsCreateTransform(hLab, TYPE_LabA_DBL, hLab, TYPE_Lab_DBL, 0, 0);
+
+	a.L = 100; a.a = 0; a.b= 0;
+	cmsDoTransform(xform, &a, &b, 1);
+
+	cmsDeleteTransform(xform);
+	cmsCloseProfile(hLab);
+}
 
 
 
@@ -1646,12 +2300,18 @@ int main(int argc, char *argv[])
 {
        int lExhaustive = 0;   
 
-       printf("little cms testbed. Ver %1.2f [build %s %s]\n\n", LCMS_VERSION / 100., __DATE__, __TIME__);
+	   // #include "crtdbg.h"
+	   // _CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF ); 
 
+	  	   
+         
+       printf("little cms testbed. Ver %1.2f [build %s %s]\n\n", LCMS_VERSION / 100., __DATE__, __TIME__);
+       
 #ifndef LCMS_DLL
 
-       if (!CheckEndianess()) return 1;
+       if (!CheckEndianess()) return 1;	   
        if (!CheckSwab()) return 1;
+	   if (!CheckQuickFloor()) return 1;
 
        TestFixedPoint();
      
@@ -1661,31 +2321,47 @@ int main(int argc, char *argv[])
        if (!TestLinearInterpolation(lExhaustive)) return 1;
        if (!TestReverseLinearInterpolation()) return 1;
 
+	   
        
 
        if (!Test3D()) return 1;
        if (!TestMatrixCreation()) return 1;
        if (!GetInfoTest()) return 1;
 
+
 #endif
 
        if (!Test_sRGB()) return 1;
+	
        if (!RealProfilesTest()) return 1;
-       if (!TestInducedError(TYPE_LABA_16)) return 1;
-
+       if (!TestInducedError(TYPE_LABA_16)) return 1;	   
        if (!TestPreview()) return 1;
-       if (!TestMultiprofile()) return 1;
+       if (!TestMultiprofile()) return 1; 	   	 
        if (!TestLinearizationDevicelink()) return 1;
-       if (!TestDeviceLinkGeneration()) return 0;
+	  	 
+       if (!TestDeviceLinkGeneration()) return 1; 	  
+	   
 
+       if (!TestLinearizationDevicelink2()) return 1;
+	  
+
+       if (!TestInkLimiting()) return 1;
+       if (!TestSaveToMem()) return 1;
+       if (!TestNamedColor()) return 1;  
+       if (!TestIT8()) return 1;
+	   if (!TestPostScript()) return 1;
+	   if (!TestColorantTableTag()) return 1;
+	   
 #ifdef ICM_COMPARATIVE
 #ifndef NON_WINDOWS
-       CompareWithICM();
+       CompareWithICM_8bit();
+       CompareWithICM_16bit();
 #endif
 #endif
 
 #ifdef CHECK_SPEED    
        SpeedTest();
+	   SpeedTest2();
 #endif
 
        printf("\nSuccess.\n");
@@ -1695,3 +2371,8 @@ int main(int argc, char *argv[])
 }
 
 
+
+
+
+
+
